@@ -5,12 +5,7 @@ from odict.pyodict import _odict
 import copy
 import pickle
 import sys
-
-
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import unittest
 
 
 class TestOdict(unittest.TestCase):
@@ -271,6 +266,7 @@ class TestOdict(unittest.TestCase):
         self.assertEqual(o.keys(), ['1', '2', '3'])
         o.alter_key('1', 'foo')
         self.assertEqual(o.keys(), ['foo', '2', '3'])
+        self.assertEqual(o.rkeys(), ['3', '2', 'foo'])
         self.assertTrue(o._dict_impl() is dict)
         dict_values = o._dict_impl().values(o)
         self.assertEqual(len(dict_values), 3)
@@ -282,6 +278,7 @@ class TestOdict(unittest.TestCase):
         self.assertEqual(o.lh, 'foo')
         o.alter_key('2', 'bar')
         self.assertEqual(o.keys(), ['foo', 'bar', '3'])
+        self.assertEqual(o.rkeys(), ['3', 'bar', 'foo'])
         dict_values = o._dict_impl().values(o)
         self.assertEqual(len(dict_values), 3)
         self.assertTrue(['bar', 'c', _nil] in dict_values)
@@ -291,6 +288,7 @@ class TestOdict(unittest.TestCase):
         self.assertEqual(o['bar'], 'b')
         o.alter_key('3', 'baz')
         self.assertEqual(o.keys(), ['foo', 'bar', 'baz'])
+        self.assertEqual(o.rkeys(), ['baz', 'bar', 'foo'])
         dict_values = o._dict_impl().values(o)
         self.assertEqual(len(dict_values), 3)
         self.assertTrue(['foo', 'b', 'baz'] in dict_values)
@@ -316,55 +314,81 @@ class TestOdict(unittest.TestCase):
         with self.assertRaises(ValueError):
             o.swap('0', '0')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case first 2, a < b
         o.swap('0', '1')
         self.assertEqual(o.keys(), ['1', '0', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '0', '1'])
         self.assertEqual(o.values(), ['b', 'a', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('1', '4'))
         # Case first 2, a > b
         o.swap('0', '1')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case last 2, a < b
         o.swap('3', '4')
         self.assertEqual(o.keys(), ['0', '1', '2', '4', '3'])
+        self.assertEqual(o.rkeys(), ['3', '4', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'e', 'd'])
+        self.assertEqual((o.lh, o.lt), ('0', '3'))
         # Case last 2, a > b
         o.swap('3', '4')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case neighbors, a < b
         o.swap('1', '2')
         self.assertEqual(o.keys(), ['0', '2', '1', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '1', '2', '0'])
         self.assertEqual(o.values(), ['a', 'c', 'b', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case neighbors, a > b
         o.swap('1', '2')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case non neighbors, one key first, a < b
         o.swap('0', '2')
         self.assertEqual(o.keys(), ['2', '1', '0', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '0', '1', '2'])
         self.assertEqual(o.values(), ['c', 'b', 'a', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('2', '4'))
         # Case non neighbors, one key first, a > b
         o.swap('0', '2')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case non neighbors, one key last, a < b
         o.swap('2', '4')
         self.assertEqual(o.keys(), ['0', '1', '4', '3', '2'])
+        self.assertEqual(o.rkeys(), ['2', '3', '4', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'e', 'd', 'c'])
+        self.assertEqual((o.lh, o.lt), ('0', '2'))
         # Case non neighbors, one key last, a > b
         o.swap('2', '4')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case non neighbors, a < b
         o.swap('1', '3')
         self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '2', '3', '0'])
         self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
         # Case non neighbors, a > b
         o.swap('1', '3')
         self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
         self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
 
     def test_insertbefore(self):
         o = odict([('0', 'a')])
@@ -374,10 +398,14 @@ class TestOdict(unittest.TestCase):
             o.insertbefore('x', '1', 'b')
         o.insertbefore('0', '1', 'b')
         self.assertEqual(o.keys(), ['1', '0'])
+        self.assertEqual(o.rkeys(), ['0', '1'])
         self.assertEqual(o.values(), ['b', 'a'])
+        self.assertEqual((o.lh, o.lt), ('1', '0'))
         o.insertbefore('0', '2', 'c')
         self.assertEqual(o.keys(), ['1', '2', '0'])
+        self.assertEqual(o.rkeys(), ['0', '2', '1'])
         self.assertEqual(o.values(), ['b', 'c', 'a'])
+        self.assertEqual((o.lh, o.lt), ('1', '0'))
 
     def test_insertafter(self):
         o = odict([('0', 'a')])
@@ -387,29 +415,207 @@ class TestOdict(unittest.TestCase):
             o.insertafter('x', '1', 'b')
         o.insertafter('0', '1', 'b')
         self.assertEqual(o.keys(), ['0', '1'])
+        self.assertEqual(o.rkeys(), ['1', '0'])
         self.assertEqual(o.values(), ['a', 'b'])
+        self.assertEqual((o.lh, o.lt), ('0', '1'))
         o.insertafter('0', '2', 'c')
         self.assertEqual(o.keys(), ['0', '2', '1'])
+        self.assertEqual(o.rkeys(), ['1', '2', '0'])
         self.assertEqual(o.values(), ['a', 'c', 'b'])
+        self.assertEqual((o.lh, o.lt), ('0', '1'))
 
     def test_insertfirst(self):
         o = odict()
         o.insertfirst('0', 'a')
         self.assertEqual(o.keys(), ['0'])
+        self.assertEqual(o.rkeys(), ['0'])
         self.assertEqual(o.values(), ['a'])
         o.insertfirst('1', 'b')
         self.assertEqual(o.keys(), ['1', '0'])
+        self.assertEqual(o.rkeys(), ['0', '1'])
         self.assertEqual(o.values(), ['b', 'a'])
 
     def test_insertlast(self):
         o = odict()
         o.insertlast('0', 'a')
         self.assertEqual(o.keys(), ['0'])
+        self.assertEqual(o.rkeys(), ['0'])
         self.assertEqual(o.values(), ['a'])
         o.insertlast('1', 'b')
         self.assertEqual(o.keys(), ['0', '1'])
+        self.assertEqual(o.rkeys(), ['1', '0'])
         self.assertEqual(o.values(), ['a', 'b'])
 
+    def test_movebefore(self):
+        o = odict([('0', 'a'), ('1', 'b'), ('2', 'c'), ('3', 'd'), ('4', 'e')])
+        with self.assertRaises(ValueError):
+            o.movebefore('0', '0')
+        # case no neighbors ref < key
+        o.movebefore('1', '3')
+        self.assertEqual(o.keys(), ['0', '3', '1', '2', '4'])
+        self.assertEqual(o.rkeys(), ['4', '2', '1', '3', '0'])
+        self.assertEqual(o.values(), ['a', 'd', 'b', 'c', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case no neighbors ref > key
+        o.movebefore('2', '3')
+        self.assertEqual(o.keys(), ['0', '1', '3', '2', '4'])
+        self.assertEqual(o.rkeys(), ['4', '2', '3', '1', '0'])
+        self.assertEqual(o.values(), ['a', 'b', 'd', 'c', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case neighbors ref < key
+        o.movebefore('3', '2')
+        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
+        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case neighbors ref > key
+        o.movebefore('3', '2')
+        self.assertEqual(o.keys(), ['0', '1', '2', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '2', '1', '0'])
+        self.assertEqual(o.values(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case move first, no neighbors
+        o.movebefore('0', '2')
+        self.assertEqual(o.keys(), ['2', '0', '1', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '1', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'b', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('2', '4'))
+        # case move first, neighbors
+        o.movebefore('2', '0')
+        self.assertEqual(o.keys(), ['0', '2', '1', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '1', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'b', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case ref last, no neighbors
+        o.movebefore('4', '1')
+        self.assertEqual(o.keys(), ['0', '2', '3', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '3', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'd', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case ref last, neighbors
+        o.movebefore('4', '1')
+        self.assertEqual(o.keys(), ['0', '2', '3', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '3', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'd', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case key first, no neighbors
+        o.movebefore('3', '0')
+        self.assertEqual(o.keys(), ['2', '0', '3', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '3', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'd', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('2', '4'))
+        # case key first, neighbors
+        o.movebefore('0', '2')
+        self.assertEqual(o.keys(), ['2', '0', '3', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '3', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'd', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('2', '4'))
+        # case key last, no neighbors
+        o.movebefore('3', '4')
+        self.assertEqual(o.keys(), ['2', '0', '4', '3', '1'])
+        self.assertEqual(o.rkeys(), ['1', '3', '4', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'e', 'd', 'b'])
+        self.assertEqual((o.lh, o.lt), ('2', '1'))
+        # case key last, neighbors
+        o.movebefore('3', '1')
+        self.assertEqual(o.keys(), ['2', '0', '4', '1', '3'])
+        self.assertEqual(o.rkeys(), ['3', '1', '4', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'e', 'b', 'd'])
+        self.assertEqual((o.lh, o.lt), ('2', '3'))
+
+    def test_moveafter(self):
+        o = odict([('0', 'a'), ('1', 'b'), ('2', 'c'), ('3', 'd'), ('4', 'e')])
+        with self.assertRaises(ValueError):
+            o.moveafter('0', '0')
+        # case no neighbors ref < key
+        o.moveafter('1', '3')
+        self.assertEqual(o.keys(), ['0', '1', '3', '2', '4'])
+        self.assertEqual(o.rkeys(), ['4', '2', '3', '1', '0'])
+        self.assertEqual(o.values(), ['a', 'b', 'd', 'c', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case no neighbors ref > key
+        o.moveafter('2', '1')
+        self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '2', '3', '0'])
+        self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case neighbors ref < key
+        o.moveafter('3', '2')
+        self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '2', '3', '0'])
+        self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case neighbors ref > key
+        o.moveafter('2', '3')
+        self.assertEqual(o.keys(), ['0', '2', '3', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '3', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'd', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case move last, no neighbors
+        o.moveafter('4', '3')
+        self.assertEqual(o.keys(), ['0', '2', '1', '4', '3'])
+        self.assertEqual(o.rkeys(), ['3', '4', '1', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'b', 'e', 'd'])
+        self.assertEqual((o.lh, o.lt), ('0', '3'))
+        # case move last, neighbors
+        o.moveafter('3', '4')
+        self.assertEqual(o.keys(), ['0', '2', '1', '3', '4'])
+        self.assertEqual(o.rkeys(), ['4', '3', '1', '2', '0'])
+        self.assertEqual(o.values(), ['a', 'c', 'b', 'd', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case ref first, no neighbors
+        o.moveafter('0', '3')
+        self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '2', '3', '0'])
+        self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case ref first, neighbors
+        o.moveafter('0', '3')
+        self.assertEqual(o.keys(), ['0', '3', '2', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '2', '3', '0'])
+        self.assertEqual(o.values(), ['a', 'd', 'c', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('0', '4'))
+        # case key first, no neighbors
+        o.moveafter('2', '0')
+        self.assertEqual(o.keys(), ['3', '2', '0', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '0', '2', '3'])
+        self.assertEqual(o.values(), ['d', 'c', 'a', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('3', '4'))
+        # case key first, neighbors
+        o.moveafter('2', '3')
+        self.assertEqual(o.keys(), ['2', '3', '0', '1', '4'])
+        self.assertEqual(o.rkeys(), ['4', '1', '0', '3', '2'])
+        self.assertEqual(o.values(), ['c', 'd', 'a', 'b', 'e'])
+        self.assertEqual((o.lh, o.lt), ('2', '4'))
+        # case key last, no neighbors
+        o.moveafter('0', '4')
+        self.assertEqual(o.keys(), ['2', '3', '0', '4', '1'])
+        self.assertEqual(o.rkeys(), ['1', '4', '0', '3', '2'])
+        self.assertEqual(o.values(), ['c', 'd', 'a', 'e', 'b'])
+        self.assertEqual((o.lh, o.lt), ('2', '1'))
+        # case key last, neighbors
+        o.moveafter('4', '1')
+        self.assertEqual(o.keys(), ['2', '3', '0', '4', '1'])
+        self.assertEqual(o.rkeys(), ['1', '4', '0', '3', '2'])
+        self.assertEqual(o.values(), ['c', 'd', 'a', 'e', 'b'])
+        self.assertEqual((o.lh, o.lt), ('2', '1'))
+
+    def test_movefirst(self):
+        o = odict([('0', 'a'), ('1', 'b'), ('2', 'c')])
+        o.movefirst('2')
+        self.assertEqual(o.keys(), ['2', '0', '1'])
+        self.assertEqual(o.rkeys(), ['1', '0', '2'])
+        self.assertEqual(o.values(), ['c', 'a', 'b'])
+        self.assertEqual((o.lh, o.lt), ('2', '1'))
+
+    def test_movelast(self):
+        o = odict([('0', 'a'), ('1', 'b'), ('2', 'c')])
+        o.movelast('0')
+        self.assertEqual(o.keys(), ['1', '2', '0'])
+        self.assertEqual(o.rkeys(), ['0', '2', '1'])
+        self.assertEqual(o.values(), ['b', 'c', 'a'])
+        self.assertEqual((o.lh, o.lt), ('1', '0'))
+
     def test_next_key(self):
         o = odict()
         with self.assertRaises(KeyError):
@@ -432,4 +638,10 @@ class TestOdict(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    unittest.main()  # pragma: no cover
+    from odict import tests
+
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.findTestCases(tests))
+    runner = unittest.TextTestRunner(failfast=True)
+    result = runner.run(suite)
+    sys.exit(not result.wasSuccessful())
